@@ -65,6 +65,10 @@
 #endif
 #endif
 
+#ifdef HAVE_RSVG
+#include <librsvg/rsvg.h>
+#endif
+
 #define EOM_IMAGE_GET_PRIVATE(object) \
 	(G_TYPE_INSTANCE_GET_PRIVATE ((object), EOM_TYPE_IMAGE, EomImagePrivate))
 
@@ -294,7 +298,7 @@ eom_image_init (EomImage *img)
 	img->priv->file_is_changed = FALSE;
 	g_mutex_init (&img->priv->status_mutex);
 	img->priv->status = EOM_IMAGE_STATUS_UNKNOWN;
-        img->priv->metadata_status = EOM_IMAGE_METADATA_NOT_READ;
+	img->priv->metadata_status = EOM_IMAGE_METADATA_NOT_READ;
 	img->priv->is_monitored = FALSE;
 	img->priv->undo_stack = NULL;
 	img->priv->trans = NULL;
@@ -478,43 +482,9 @@ do_emit_size_prepared_signal (EomImage *img)
 static void
 eom_image_emit_size_prepared (EomImage *img)
 {
-	gdk_threads_add_idle_full (G_PRIORITY_DEFAULT_IDLE,
-				   (GSourceFunc) do_emit_size_prepared_signal,
-				   g_object_ref (img), g_object_unref);
-}
-
-static gboolean
-check_loader_threadsafety (GdkPixbufLoader *loader, gboolean *result)
-{
-	GdkPixbufFormat *format;
-	gboolean ret_val = FALSE;
-
-	format = gdk_pixbuf_loader_get_format (loader);
-	if (format) {
-		ret_val = TRUE;
-		if (result)
-		/* FIXME: We should not be accessing this struct internals
- 		 * directly. Keep track of bug #469209 to fix that. */
-			*result = format->flags & GDK_PIXBUF_FORMAT_THREADSAFE;
-	}
-
-	return ret_val;
-}
-
-static void
-eom_image_pre_size_prepared (GdkPixbufLoader *loader,
-			     gint width,
-			     gint height,
-			     gpointer data)
-{
-	EomImage *img;
-
-	eom_debug (DEBUG_IMAGE_LOAD);
-
-	g_return_if_fail (EOM_IS_IMAGE (data));
-
-	img = EOM_IMAGE (data);
-	check_loader_threadsafety (loader, &img->priv->threadsafe_format);
+	g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
+	                (GSourceFunc) do_emit_size_prepared_signal,
+	                 g_object_ref (img), g_object_unref);
 }
 
 static void
@@ -539,9 +509,7 @@ eom_image_size_prepared (GdkPixbufLoader *loader,
 	g_mutex_unlock (&img->priv->status_mutex);
 
 #ifdef HAVE_EXIF
-	if (img->priv->threadsafe_format && (!img->priv->autorotate || img->priv->exif))
-#else
-	if (img->priv->threadsafe_format)
+	if (!img->priv->autorotate || img->priv->exif)
 #endif
 		eom_image_emit_size_prepared (img);
 }
@@ -895,6 +863,7 @@ eom_image_real_load (EomImage *img,
 	gboolean failed = FALSE;
 	gboolean first_run = TRUE;
 	gboolean set_metadata = TRUE;
+        gboolean use_rsvg = FALSE;
 	gboolean read_image_data = (data2read & EOM_IMAGE_DATA_IMAGE);
 	gboolean read_only_dimension = (data2read & EOM_IMAGE_DATA_DIMENSION) &&
 				  ((data2read ^ EOM_IMAGE_DATA_DIMENSION) == 0);
@@ -909,8 +878,6 @@ eom_image_real_load (EomImage *img,
 		priv->file_type = NULL;
 	}
 
-	priv->threadsafe_format = FALSE;
-
 	eom_image_get_file_info (img, &priv->bytes, &mime_type, error);
 
 	if (error && *error) {
@@ -953,51 +920,44 @@ eom_image_real_load (EomImage *img,
 	buffer = g_new0 (guchar, EOM_IMAGE_READ_BUFFER_SIZE);
 
 	if (read_image_data || read_only_dimension) {
-		gboolean checked_threadsafety = FALSE;
-
 #ifdef HAVE_RSVG
 		if (priv->svg != NULL) {
 			g_object_unref (priv->svg);
 			priv->svg = NULL;
 		}
 
-		if (!strcmp (mime_type, "image/svg+xml")) {
+		if (!strcmp (mime_type, "image/svg+xml")
+#if LIBRSVG_CHECK_FEATURE(SVGZ)
+                    || !strcmp (mime_type, "image/svg+xml-compressed")
+#endif
+                ) {
 			gchar *file_path;
 			/* Keep the object for rendering */
 			priv->svg = rsvg_handle_new ();
+                        use_rsvg = (priv->svg != NULL);
 			file_path = g_file_get_path (priv->file);
 			rsvg_handle_set_base_uri (priv->svg, file_path);
 			g_free (file_path);
 		}
 #endif
-		loader = gdk_pixbuf_loader_new_with_mime_type (mime_type, error);
 
-		if (error && *error) {
-			g_error_free (*error);
-			*error = NULL;
+                if (!use_rsvg) {
+		        loader = gdk_pixbuf_loader_new_with_mime_type (mime_type, error);
 
-			loader = gdk_pixbuf_loader_new ();
-		} else {
-			/* The mimetype-based loader should know the
-			 * format here already. */
-			checked_threadsafety = check_loader_threadsafety (loader, &priv->threadsafe_format);
-		}
+		        if (error && *error) {
+			        g_error_free (*error);
+			        *error = NULL;
 
-		/* This is used to detect non-threadsafe loaders and disable
- 		 * any possible asyncronous task that could bring deadlocks
- 		 * to image loading process. */
-		if (!checked_threadsafety)
-			g_signal_connect (loader,
-					  "size-prepared",
-					  G_CALLBACK (eom_image_pre_size_prepared),
-					  img);
+			        loader = gdk_pixbuf_loader_new ();
+		        }
 
-		g_signal_connect_object (G_OBJECT (loader),
-					 "size-prepared",
-					 G_CALLBACK (eom_image_size_prepared),
-					 img,
-					 0);
-        }
+		        g_signal_connect_object (G_OBJECT (loader),
+					         "size-prepared",
+					         G_CALLBACK (eom_image_size_prepared),
+					         img,
+					         0);
+                 }
+	}
 	g_free (mime_type);
 
 	while (!priv->cancel_loading) {
@@ -1022,17 +982,23 @@ eom_image_real_load (EomImage *img,
 		}
 
 		if ((read_image_data || read_only_dimension)) {
+#ifdef HAVE_RSVG
+			if (use_rsvg) {
+                            gboolean res;
+
+			    res = rsvg_handle_write (priv->svg, buffer,
+                                                     bytes_read, error);
+
+                            if (G_UNLIKELY (!res)) {
+				failed = TRUE;
+				break;
+                            }
+			} else
+#endif
 			if (!gdk_pixbuf_loader_write (loader, buffer, bytes_read, error)) {
 				failed = TRUE;
 				break;
 			}
-#ifdef HAVE_RSVG
-			if (eom_image_is_svg (img) &&
-			    !rsvg_handle_write (priv->svg, buffer, bytes_read, error)) {
-				failed = TRUE;
-				break;
-			}
-#endif
 		}
 
 		bytes_read_total += bytes_read;
@@ -1049,16 +1015,15 @@ eom_image_real_load (EomImage *img,
 				if (data2read == EOM_IMAGE_DATA_EXIF) {
 					g_set_error (error,
 						     EOM_IMAGE_ERROR,
-                                	             EOM_IMAGE_ERROR_GENERIC,
-                                	             _("EXIF not supported for this file format."));
+						     EOM_IMAGE_ERROR_GENERIC,
+						     _("EXIF not supported for this file format."));
 					break;
 				}
 
-				if (priv->threadsafe_format)
-					eom_image_emit_size_prepared (img);
+				eom_image_emit_size_prepared (img);
 
-                                priv->metadata_status = EOM_IMAGE_METADATA_NOT_AVAILABLE;
-                        }
+				priv->metadata_status = EOM_IMAGE_METADATA_NOT_AVAILABLE;
+			}
 
 			first_run = FALSE;
 		}
@@ -1078,7 +1043,7 @@ eom_image_real_load (EomImage *img,
 					eom_image_set_xmp_data (img, md_reader);
 #endif
 					set_metadata = FALSE;
-                                        priv->metadata_status = EOM_IMAGE_METADATA_READY;
+					priv->metadata_status = EOM_IMAGE_METADATA_READY;
 				}
 
 				if (data2read == EOM_IMAGE_DATA_EXIF)
@@ -1093,6 +1058,14 @@ eom_image_real_load (EomImage *img,
 	}
 
 	if (read_image_data || read_only_dimension) {
+#ifdef HAVE_RSVG
+		if (use_rsvg) {
+			/* Ignore the error if loading failed earlier
+			 * as the error will already be set in that case */
+			rsvg_handle_close (priv->svg,
+			                   (failed ? NULL : error));
+                } else
+#endif
 		if (failed) {
 			gdk_pixbuf_loader_close (loader, NULL);
 		} else if (!gdk_pixbuf_loader_close (loader, error)) {
@@ -1101,12 +1074,8 @@ eom_image_real_load (EomImage *img,
 				 * images as well. */
 				g_clear_error (error);
 			}
-	        }
-#ifdef HAVE_RSVG
-		if (eom_image_is_svg (img))
-			rsvg_handle_close (priv->svg, error);
-#endif
-        }
+		}
+	}
 
 	g_free (buffer);
 
@@ -1129,6 +1098,14 @@ eom_image_real_load (EomImage *img,
 			g_object_unref (priv->image);
 		}
 
+#ifdef HAVE_RSVG
+                if (use_rsvg) {
+                    priv->image = rsvg_handle_get_pixbuf (priv->svg);
+                } else
+#endif
+                
+                {
+
 		priv->anim = gdk_pixbuf_loader_get_animation (loader);
 
 		if (gdk_pixbuf_animation_is_static_image (priv->anim)) {
@@ -1139,24 +1116,27 @@ eom_image_real_load (EomImage *img,
 			priv->image = gdk_pixbuf_animation_iter_get_pixbuf (priv->anim_iter);
 		}
 
+                }
+
 		if (G_LIKELY (priv->image != NULL)) {
-			g_object_ref (priv->image);
+                        if (!use_rsvg)
+			        g_object_ref (priv->image);
 
 			priv->width = gdk_pixbuf_get_width (priv->image);
 			priv->height = gdk_pixbuf_get_height (priv->image);
 
-			format = gdk_pixbuf_loader_get_format (loader);
+                        if (use_rsvg) {
+                                format = NULL;
+                                priv->file_type = g_strdup ("svg");
+                        } else {
+			        format = gdk_pixbuf_loader_get_format (loader);
+                        }
 
 			if (format != NULL) {
 				priv->file_type = gdk_pixbuf_format_get_name (format);
 			}
 
 			priv->file_is_changed = FALSE;
-
-			/* If it's non-threadsafe loader, then trigger window
- 			 * showing in the end of the process. */
-			if (!priv->threadsafe_format)
-				eom_image_emit_size_prepared (img);
 		} else {
 			/* Some loaders don't report errors correctly.
 			 * Error will be set below. */
@@ -1307,6 +1287,14 @@ eom_image_set_thumbnail (EomImage *img, GdkPixbuf *thumbnail)
 	}
 }
 
+/**
+ * eom_image_get_pixbuf:
+ * @img: a #EomImage
+ *
+ * Gets the #GdkPixbuf of the image
+ *
+ * Returns: (transfer full): a #GdkPixbuf
+ **/
 GdkPixbuf *
 eom_image_get_pixbuf (EomImage *img)
 {
@@ -1335,6 +1323,14 @@ eom_image_get_profile (EomImage *img)
 }
 #endif
 
+/**
+ * eom_image_get_thumbnail:
+ * @img: a #EomImage
+ *
+ * Gets the thumbnail pixbuf for @img
+ *
+ * Returns: (transfer full): a #GdkPixbuf with a thumbnail
+ **/
 GdkPixbuf *
 eom_image_get_thumbnail (EomImage *img)
 {
@@ -2014,6 +2010,15 @@ eom_image_get_exif_info (EomImage *img)
 }
 #endif
 
+/**
+ * eom_image_get_xmp_info:
+ * @img: a #EomImage
+ *
+ * Gets the XMP info for @img or NULL if compiled without
+ * libexempi support.
+ *
+ * Returns: (transfer full): the xmp data
+ **/
 gpointer
 eom_image_get_xmp_info (EomImage *img)
 {
@@ -2034,6 +2039,14 @@ eom_image_get_xmp_info (EomImage *img)
 }
 
 
+/**
+ * eom_image_get_file:
+ * @img: a #EomImage
+ *
+ * Gets the #GFile associated with @img
+ *
+ * Returns: (transfer full): a #GFile
+ **/
 GFile *
 eom_image_get_file (EomImage *img)
 {
@@ -2109,9 +2122,9 @@ eom_image_get_status (EomImage *img)
 EomImageMetadataStatus
 eom_image_get_metadata_status (EomImage *img)
 {
-        g_return_val_if_fail (EOM_IS_IMAGE (img), EOM_IMAGE_METADATA_NOT_AVAILABLE);
+	g_return_val_if_fail (EOM_IS_IMAGE (img), EOM_IMAGE_METADATA_NOT_AVAILABLE);
 
-        return img->priv->metadata_status;
+	return img->priv->metadata_status;
 }
 
 void
@@ -2155,6 +2168,13 @@ compare_quarks (gconstpointer a, gconstpointer b)
 	return quark - GPOINTER_TO_INT (b);
 }
 
+/**
+ * eom_image_get_supported_mime_types:
+ *
+ * Gets the list of supported mimetypes
+ *
+ * Returns: (transfer none)(element-type utf8): a #GList of supported mimetypes
+ **/
 GList *
 eom_image_get_supported_mime_types (void)
 {
@@ -2219,8 +2239,7 @@ eom_image_iter_advance (EomImage *img)
 
 	priv = img->priv;
 
-	if ((new_frame = gdk_pixbuf_animation_iter_advance (img->priv->anim_iter, NULL)) == TRUE)
-	  {      
+	if ((new_frame = gdk_pixbuf_animation_iter_advance (img->priv->anim_iter, NULL)) == TRUE) {      
 		g_mutex_lock (&priv->status_mutex);
 		g_object_unref (priv->image);
 		priv->image = gdk_pixbuf_animation_iter_get_pixbuf (priv->anim_iter);
@@ -2232,12 +2251,12 @@ eom_image_iter_advance (EomImage *img)
 			priv->image = transformed;
 			priv->width = gdk_pixbuf_get_width (transformed);
 			priv->height = gdk_pixbuf_get_height (transformed);
-		}      
+		}
 		g_mutex_unlock (&priv->status_mutex);
 		/* Emit next frame signal so we can update the display */
 		g_signal_emit (img, signals[SIGNAL_NEXT_FRAME], 0,
 			       gdk_pixbuf_animation_iter_get_delay_time (priv->anim_iter));
-	  }
+	}
 
 	return new_frame;
 }
@@ -2321,6 +2340,8 @@ eom_image_get_svg (EomImage *img)
 	return img->priv->svg;
 }
 
+#endif
+
 EomTransform *
 eom_image_get_transform (EomImage *img)
 {
@@ -2329,7 +2350,13 @@ eom_image_get_transform (EomImage *img)
 	return img->priv->trans;
 }
 
-#endif
+EomTransform*
+eom_image_get_autorotate_transform (EomImage *img)
+{
+	g_return_val_if_fail (EOM_IS_IMAGE (img), NULL);
+
+	return img->priv->trans_autorotate;
+}
 
 /**
  * eom_image_file_changed:
@@ -2354,3 +2381,12 @@ eom_image_is_file_changed (EomImage *img)
 
 	return img->priv->file_is_changed;
 }
+
+gboolean
+eom_image_is_jpeg (EomImage *img)
+{
+	g_return_val_if_fail (EOM_IS_IMAGE (img), FALSE);
+
+	return ((img->priv->file_type != NULL) && (g_ascii_strcasecmp (img->priv->file_type, EOM_FILE_FORMAT_JPEG) == 0));
+}
+
