@@ -36,12 +36,6 @@
 #include "eom-print-image-setup.h"
 #include "eom-print-preview.h"
 
-#if !GTK_CHECK_VERSION(2,24,0)
-	#define gtk_combo_box_text_new() gtk_combo_box_new_text()
-	#define gtk_combo_box_text_insert_text(x,y,z) gtk_combo_box_insert_text(x,y,z)
-	#define GTK_COMBO_BOX_TEXT(x) GTK_COMBO_BOX(x)
-#endif
-
 /**
  * SECTION:
  * @Title: Printing Custom Widget
@@ -57,9 +51,9 @@
 #define EOM_PRINT_IMAGE_SETUP_GET_PRIVATE(object) \
 	(G_TYPE_INSTANCE_GET_PRIVATE ((object), EOM_TYPE_PRINT_IMAGE_SETUP, EomPrintImageSetupPrivate))
 
-G_DEFINE_TYPE (EomPrintImageSetup, eom_print_image_setup, GTK_TYPE_TABLE);
+G_DEFINE_TYPE (EomPrintImageSetup, eom_print_image_setup, GTK_TYPE_GRID);
 
-struct EomPrintImageSetupPrivate {
+struct _EomPrintImageSetupPrivate {
 	GtkWidget *left;
 	GtkWidget *right;
 	GtkWidget *top;
@@ -645,49 +639,47 @@ static GtkWidget *
 wrap_in_frame (const gchar *label,
                GtkWidget   *child)
 {
-	GtkWidget *frame, *alignment, *label_widget;
+	GtkWidget *frame, *label_widget;
 	gchar *bold_text;
 
 	label_widget = gtk_label_new ("");
-	gtk_misc_set_alignment (GTK_MISC (label_widget), 0.0, 0.5);
+	gtk_widget_set_halign (label_widget, GTK_ALIGN_START);
 	gtk_widget_show (label_widget);
 
 	bold_text = g_markup_printf_escaped ("<b>%s</b>", label);
 	gtk_label_set_markup (GTK_LABEL (label_widget), bold_text);
 	g_free (bold_text);
 
-	frame = gtk_vbox_new (FALSE, 6);
+	frame = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 	gtk_box_pack_start (GTK_BOX (frame), label_widget, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (frame), child, FALSE, FALSE, 0);
 
-	alignment = gtk_alignment_new (0.0, 0.0, 1.0, 1.0);
-	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment),
-				   0, 0, 12, 0);
-	gtk_box_pack_start (GTK_BOX (frame), alignment, FALSE, FALSE, 0);
-
-	gtk_container_add (GTK_CONTAINER (alignment), child);
+	gtk_widget_set_margin_start (child, 12);
 
 	gtk_widget_show (frame);
-	gtk_widget_show (alignment);
 
 	return frame;
 }
 
 static GtkWidget *
-table_attach_spin_button_with_label (GtkWidget *table,
+grid_attach_spin_button_with_label (GtkWidget *grid,
 				     const gchar* text_label,
 				     gint left, gint top)
 {
 	GtkWidget *label, *spin_button;
 
 	label = gtk_label_new_with_mnemonic (text_label);
+#if GTK_CHECK_VERSION (3, 16, 0)
+	gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+#else
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+#endif
 	spin_button = gtk_spin_button_new_with_range (0, 100, 0.01);
 	gtk_spin_button_set_digits (GTK_SPIN_BUTTON (spin_button), 2);
 	gtk_entry_set_width_chars (GTK_ENTRY (spin_button), 6);
-	gtk_table_attach (GTK_TABLE (table), label, left, left + 1,
-			  top, top + 1, GTK_FILL, GTK_FILL, 0, 0);
-	gtk_table_attach (GTK_TABLE (table), spin_button, left + 1, left + 2,
-			  top, top + 1, GTK_FILL, GTK_FILL, 0, 0);
+	gtk_grid_attach (GTK_GRID (grid), label, left, top, 1, 1);
+	gtk_grid_attach_next_to (GTK_GRID (grid), spin_button, label,
+							 GTK_POS_RIGHT, 1, 1);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin_button);
 
 	return spin_button;
@@ -856,7 +848,7 @@ static void
 eom_print_image_setup_init (EomPrintImageSetup *setup)
 {
 	GtkWidget *frame;
-	GtkWidget *table;
+	GtkWidget *grid;
 	GtkWidget *label;
 	GtkWidget *hscale;
 	GtkWidget *combobox;
@@ -870,21 +862,26 @@ eom_print_image_setup_init (EomPrintImageSetup *setup)
 
 	priv->image = NULL;
 
-	table = gtk_table_new (3, 4, FALSE);
-	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
-	gtk_table_set_col_spacings (GTK_TABLE (table), 12);
-	frame = wrap_in_frame (_("Position"), table);
-	gtk_table_attach (GTK_TABLE (setup), frame,
-			  0, 1, 0, 1,  GTK_FILL, 0,
-			  0, 0);
+	grid = gtk_grid_new ();
+	gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
+	gtk_grid_set_column_spacing (GTK_GRID (grid), 12);
+	frame = wrap_in_frame (_("Position"), grid);
+	gtk_grid_attach (GTK_GRID (setup), frame, 0, 0, 1, 1);
 
-	priv->left = table_attach_spin_button_with_label (table, _("_Left:"), 0, 0);
-	priv->right = table_attach_spin_button_with_label (table, _("_Right:"), 0, 1);
-	priv->top = table_attach_spin_button_with_label (table, _("_Top:"), 2, 0);
-	priv->bottom = table_attach_spin_button_with_label (table, _("_Bottom:"), 2, 1);
+	priv->left = grid_attach_spin_button_with_label (grid,
+													 _("_Left:"), 0, 0);
+	priv->right = grid_attach_spin_button_with_label (grid,
+													  _("_Right:"), 0, 1);
+	priv->top = grid_attach_spin_button_with_label (grid, _("_Top:"), 2, 0);
+	priv->bottom = grid_attach_spin_button_with_label (grid, _("_Bottom:"),
+													   2, 1);
 
 	label = gtk_label_new_with_mnemonic (_("C_enter:"));
+#if GTK_CHECK_VERSION (3, 16, 0)
+	gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+#else
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+#endif
 
 	combobox = gtk_combo_box_text_new ();
 	gtk_combo_box_text_insert_text (GTK_COMBO_BOX_TEXT (combobox),
@@ -896,45 +893,45 @@ eom_print_image_setup_init (EomPrintImageSetup *setup)
 	gtk_combo_box_text_insert_text (GTK_COMBO_BOX_TEXT (combobox),
 				   CENTER_BOTH, _("Both"));
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combobox), CENTER_NONE);
-	gtk_table_attach (GTK_TABLE (table), label,
-			  0, 1, 2, 3, GTK_FILL, GTK_FILL,
-			  0, 0);
-	gtk_table_attach (GTK_TABLE (table), combobox,
-			  1, 4, 2, 3, GTK_FILL | GTK_EXPAND, GTK_FILL,
-			  0, 0);
+	/* Attach combobox below right margin spinbutton and span until end */
+	gtk_grid_attach_next_to (GTK_GRID (grid), combobox, priv->right,
+							 GTK_POS_BOTTOM, 3, 1);
+	/* Attach the label to the left of the combobox */
+	gtk_grid_attach_next_to (GTK_GRID (grid), label, combobox, GTK_POS_LEFT,
+							 1, 1);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), combobox);
 	priv->center = combobox;
 	g_signal_connect (G_OBJECT (combobox), "changed",
 			  G_CALLBACK (on_center_changed), setup);
 
-	table = gtk_table_new (3, 4, FALSE);
-	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
-	gtk_table_set_col_spacings (GTK_TABLE (table), 12);
-	frame = wrap_in_frame (_("Size"), table);
-	gtk_table_attach (GTK_TABLE (setup), frame,
-			  0, 1, 1, 2,  GTK_FILL, 0,
-			  0, 0);
+	grid = gtk_grid_new ();
+	gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
+	gtk_grid_set_column_spacing (GTK_GRID (grid), 12);
+	frame = wrap_in_frame (_("Size"), grid);
+	gtk_grid_attach (GTK_GRID (setup), frame, 0, 1, 1, 1);
 
-	priv->width = table_attach_spin_button_with_label (table, _("_Width:"),
+	priv->width = grid_attach_spin_button_with_label (grid, _("_Width:"),
 							   0, 0);
-	priv->height = table_attach_spin_button_with_label (table, _("_Height:"),
+	priv->height = grid_attach_spin_button_with_label (grid, _("_Height:"),
 							    2, 0);
 
 	label = gtk_label_new_with_mnemonic (_("_Scaling:"));
-	hscale = gtk_hscale_new_with_range (1, 100, 1);
+	hscale = gtk_scale_new_with_range (GTK_ORIENTATION_HORIZONTAL, 1, 100, 1);
 	gtk_scale_set_value_pos (GTK_SCALE (hscale), GTK_POS_RIGHT);
 	gtk_range_set_value (GTK_RANGE (hscale), 100);
-	gtk_table_attach (GTK_TABLE (table), label,
-			  0, 1, 1, 2, GTK_FILL, GTK_FILL,
-			  0, 0);
-	gtk_table_attach (GTK_TABLE (table), hscale,
-			  1, 4, 1, 2, GTK_FILL | GTK_EXPAND, GTK_FILL,
-			  0, 0);
+	gtk_grid_attach_next_to (GTK_GRID (grid), hscale, priv->width,
+							 GTK_POS_BOTTOM, 3, 1);
+	gtk_grid_attach_next_to (GTK_GRID (grid), label, hscale, GTK_POS_LEFT,
+							 1, 1);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), hscale);
 	priv->scaling = hscale;
 
 	label = gtk_label_new_with_mnemonic (_("_Unit:"));
+#if GTK_CHECK_VERSION (3, 16, 0)
+	gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+#else
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+#endif
 
 	combobox = gtk_combo_box_text_new ();
 	gtk_combo_box_text_insert_text (GTK_COMBO_BOX_TEXT (combobox), UNIT_MM,
@@ -954,12 +951,10 @@ eom_print_image_setup_init (EomPrintImageSetup *setup)
 		set_scale_unit (setup, GTK_UNIT_MM);
 	}
 
-	gtk_table_attach (GTK_TABLE (table), label,
-			  0, 1, 2, 3, GTK_FILL, GTK_FILL,
-			  0, 0);
-	gtk_table_attach (GTK_TABLE (table), combobox,
-			  1, 4, 2, 3, GTK_FILL | GTK_EXPAND, GTK_FILL,
-			  0, 0);
+	gtk_grid_attach_next_to (GTK_GRID (grid), combobox, hscale,
+							 GTK_POS_BOTTOM, 3, 1);
+	gtk_grid_attach_next_to (GTK_GRID (grid), label, combobox, GTK_POS_LEFT,
+							 1, 1);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), combobox);
 	priv->unit = combobox;
 	g_signal_connect (G_OBJECT (combobox), "changed",
@@ -971,9 +966,8 @@ eom_print_image_setup_init (EomPrintImageSetup *setup)
 	gtk_widget_set_size_request (priv->preview, 250, 250);
 
 	frame = wrap_in_frame (_("Preview"), priv->preview);
-	gtk_table_attach (GTK_TABLE (setup), frame,
-			  1, 2, 0, 2, GTK_FILL, GTK_FILL,
-			  0, 0);
+	/* The preview widget needs to span the whole grid height */
+	gtk_grid_attach (GTK_GRID (setup), frame, 1, 0, 1, 2);
 
 	gtk_widget_show_all (GTK_WIDGET (setup));
 }
@@ -998,9 +992,7 @@ eom_print_image_setup_new (EomImage *image, GtkPageSetup *page_setup)
 	GtkWidget *preview;
 
 	setup = g_object_new (EOM_TYPE_PRINT_IMAGE_SETUP,
-			     "n-rows", 2,
-			     "n-columns", 2,
-			     "homogeneous", FALSE,
+			     "orientation", GTK_ORIENTATION_VERTICAL,
 			     "row-spacing", 18,
 			     "column-spacing", 18,
 			     "border-width", 12,
